"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShopifyHmacGuard = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const shopify_api_1 = require("@shopify/shopify-api");
const crypto_1 = require("crypto");
const core_decorators_1 = require("../core.decorators");
const hmac_constants_1 = require("./hmac.constants");
const hmac_enums_1 = require("./hmac.enums");
let ShopifyHmacGuard = class ShopifyHmacGuard {
    constructor(reflector, shopifyApi) {
        this.reflector = reflector;
        this.shopifyApi = shopifyApi;
    }
    canActivate(context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const hmacType = this.getShopifyHmacTypeFromContext(context);
            if (!hmacType) {
                return true;
            }
            const req = context
                .switchToHttp()
                .getRequest();
            switch (hmacType) {
                case hmac_enums_1.ShopifyHmacType.Query:
                    return this.validateHmacQuery(req);
                case hmac_enums_1.ShopifyHmacType.Header:
                    return this.validateHmacHeader(req);
            }
        });
    }
    validateHmacHeader(req) {
        const expectedHmac = this.getHmacFromHeaders(req);
        if (!req.rawBody) {
            throw new common_1.InternalServerErrorException(`Missing raw body in request. Ensure that 'rawBody' option is set when initializing Nest application.`);
        }
        const generatedHash = (0, crypto_1.createHmac)('sha256', this.shopifyApi.config.apiSecretKey)
            .update(req.rawBody)
            .digest('base64');
        const generatedHashBuffer = Buffer.from(generatedHash);
        const hmacBuffer = Buffer.from(expectedHmac);
        if (generatedHashBuffer.length !== hmacBuffer.length) {
            throw new common_1.UnauthorizedException('Webhook HMAC validation failed.');
        }
        if (!(0, crypto_1.timingSafeEqual)(generatedHashBuffer, hmacBuffer)) {
            throw new common_1.UnauthorizedException('Webhook HMAC validation failed.');
        }
        return true;
    }
    validateHmacQuery(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = req.query;
            try {
                if (yield this.shopifyApi.utils.validateHmac(query)) {
                    return true;
                }
                throw new shopify_api_1.InvalidHmacError('Invalid HMAC provided in query params');
            }
            catch (err) {
                throw new common_1.UnauthorizedException(err);
            }
        });
    }
    getHmacFromHeaders(req) {
        const hmacHeader = req.headers[shopify_api_1.ShopifyHeader.Hmac] ||
            req.headers[shopify_api_1.ShopifyHeader.Hmac.toLowerCase()];
        if (!hmacHeader) {
            throw new common_1.BadRequestException(`Missing required HTTP header: ${shopify_api_1.ShopifyHeader.Hmac}`);
        }
        if (typeof hmacHeader !== 'string') {
            throw new common_1.BadRequestException(`Malformed '${shopify_api_1.ShopifyHeader.Hmac}' provided: ${hmacHeader}`);
        }
        return hmacHeader;
    }
    getShopifyHmacTypeFromContext(ctx) {
        return this.reflector.getAllAndOverride(hmac_constants_1.SHOPIFY_HMAC_KEY, [ctx.getHandler(), ctx.getClass()]);
    }
};
exports.ShopifyHmacGuard = ShopifyHmacGuard;
exports.ShopifyHmacGuard = ShopifyHmacGuard = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(1, (0, core_decorators_1.InjectShopify)()),
    tslib_1.__metadata("design:paramtypes", [core_1.Reflector, Object])
], ShopifyHmacGuard);
//# sourceMappingURL=hmac.guard.js.map